@@ -1,5 +1,5 @@
 package YAML;
-our $VERSION = '1.20';
+our $VERSION = '1.21';
 
 use YAML::Mo;
 
@@ -7,8 +7,17 @@ use Exporter;
 push @YAML::ISA, 'Exporter';
 our @EXPORT = qw{ Dump Load };
 our @EXPORT_OK = qw{ freeze thaw DumpFile LoadFile Bless Blessed };
+our (
+    $UseCode, $DumpCode, $LoadCode,
+    $SpecVersion,
+    $UseHeader, $UseVersion, $UseBlock, $UseFold, $UseAliases,
+    $Indent, $SortKeys, $Preserve,
+    $AnchorPrefix, $CompressSeries, $InlineSeries, $Purity, $Stringify,
+);
+
 
 use YAML::Node; # XXX This is a temp fix for Module::Build
+use Scalar::Util qw/ openhandle /;
 
 # XXX This VALUE nonsense needs to go.
 use constant VALUE => "\x07YAML\x07VALUE\x07";
@@ -44,7 +53,7 @@ sub Load {
 sub DumpFile {
     my $OUT;
     my $filename = shift;
-    if (ref $filename eq 'GLOB') {
+    if (openhandle $filename) {
         $OUT = $filename;
     }
     else {
@@ -70,7 +79,7 @@ sub DumpFile {
 sub LoadFile {
     my $IN;
     my $filename = shift;
-    if (ref $filename eq 'GLOB') {
+    if (openhandle $filename) {
         $IN = $filename;
     }
     else {
